@@ -2,11 +2,11 @@
     Campaign state manager module
 """
 from datetime import datetime
-import sdc_helpers.utils as utils
 import boto3
 from botocore.exceptions import ClientError as botocore_clienterror
-from sagemaker.utils import name_from_base
+import sdc_helpers.utils as utils
 from sdc_engine_helpers.date_utils import DateUtils
+from sdc_engine_helpers.sagemaker.utils import name_from_base
 
 class CampaignStateManager:
     """
@@ -74,8 +74,11 @@ class CampaignStateManager:
     def __init__(self):
         self.sagemaker = boto3.client('sagemaker')
 
-   # pylint: disable=invalid-name
-    def sagemaker_create_model(self, training_job_name: str) -> str:
+    def sagemaker_create_model(
+            self,
+            training_job_name: str,
+            container_image: str = None
+    ) -> str:
         """
             Create model from a trained model job using Sagemaker API.
             if model exists, creation is skipped.
@@ -102,6 +105,18 @@ class CampaignStateManager:
             'ModelDataUrl': training_job_config['ModelArtifacts']['S3ModelArtifacts']
         }
 
+        if container_image is not None:
+            # override Image with provided image
+            print((
+                "Updating serving image to provided "
+                "container image, {}".format(container_image)
+            ))
+            primary_container.update(
+                {
+                    'Image': container_image
+                }
+            )
+
         model_tags = self.resource_tags
         model_tags.append({
             'Key': 'status',
@@ -125,6 +140,7 @@ class CampaignStateManager:
                         .format(model_name)
                     ))
                 else:
+                    # re-throw error
                     raise botocore_clienterror(
                         error_response=exception.operation_name,
                         operation_name=exception.response
@@ -132,7 +148,12 @@ class CampaignStateManager:
 
         return model_name
 
-    def sagemaker_update_campaign(self, endpoint_name: str, training_job_name: str):
+    def sagemaker_update_campaign(
+            self,
+            endpoint_name: str,
+            training_job_name: str,
+            container_image: str = None
+    ):
         """
             (Call vendor API) Creates a endpoint config
             and updates endpoint with this config
@@ -155,7 +176,10 @@ class CampaignStateManager:
             EndpointName=endpoint_name
         )
 
-        model_name = self.sagemaker_create_model(training_job_name=training_job_name)
+        model_name = self.sagemaker_create_model(
+            training_job_name=training_job_name,
+            container_image=container_image
+        )
 
         previous_endpoint_config = self.sagemaker.describe_endpoint_config(
             EndpointConfigName=endpoint['EndpointConfigName']
@@ -292,6 +316,12 @@ class CampaignStateManager:
             })
 
         else:
+            # Check if the database arn needs to be updated
+            arn = campaign.get('solution_version_arn', None)
+
+            vendor_campaign_solution_version_arn = self.get_vendor_campaign_solution_version_arn(
+                vendor_campaign=vendor_campaign
+            )
             # Check if the database status needs to be updated
             status = campaign.get('status', None)
 
@@ -299,12 +329,26 @@ class CampaignStateManager:
                 vendor_campaign=vendor_campaign
             )
 
-            # get solution_version_arn from vendor campaign
-            vendor_campaign_solution_version_arn = self.get_vendor_campaign_solution_version_arn(
-                vendor_campaign=vendor_campaign
-            )
-
-            if vendor_campaign_status != status:
+            if (
+                    (vendor_campaign_status != status) &
+                    (vendor_campaign_status in self.campaign_statuses['blocking'])
+            ):
+                # update status, when it differs from vendor during updating/blocking status
+                # get current refresh time, for keeping track
+                current_time_at_update = datetime.strftime(
+                    datetime.now(),
+                    self.date_utils.get_mysql_date_format()
+                )
+                state['should_refresh_status'] = True
+                state['new_status'] = vendor_campaign_status
+                state['latest_solution_version_arn'] = vendor_campaign_solution_version_arn
+                state['last_refreshed_at'] = current_time_at_update
+            elif (
+                    (vendor_campaign_solution_version_arn != arn) &
+                    (vendor_campaign_status not in self.campaign_statuses['blocking'])
+            ):
+                # when not in updating/blocking status, and db solution arn is mismatched
+                # with vendor solution arn
                 # get current refresh time, for keeping track
                 current_time_at_update = datetime.strftime(
                     datetime.now(),
@@ -403,23 +447,15 @@ class CampaignStateManager:
             'last_updated_at': None
         }
 
-        # get vendor current campaign
-        vendor_campaign = self.sagemaker.describe_endpoint(
-            EndpointName=campaign.get('arn')
-        )
-
-        # get vendor current solution
-        vendor_solution = self.sagemaker.describe_training_job(
-            TrainingJobName=solution.get('arn')
-        )
-
         # run vendor campaign update workflow
         response = self.sagemaker_update_campaign(
-            endpoint_name=vendor_campaign.get('EndpointName'),
-            training_job_name=vendor_solution.get('TrainingJobName')
+            endpoint_name=campaign.get('arn'),
+            training_job_name=solution.get('arn'),
+            container_image=campaign.get('container_image', None)
         )
+
         new_campaign['arn'] = response.get('EndpointName')
-        new_campaign['solution_version_arn'] = vendor_solution.get('TrainingJobName')
+        new_campaign['solution_version_arn'] = response.get('TrainingJobName')
 
         # get the current time as last updated at time
         current_time_at_update = datetime.strftime(
