@@ -12,7 +12,11 @@
 import boto3
 from sqlalchemy.exc import InternalError
 from botocore.exceptions import ClientError as botocore_clienterror
-from sdc_engine_helpers import decorators
+from sdc_helpers.slack_helper import SlackHelper
+from sdc_helpers import decorators as global_decorators
+from sdc_engine_helpers.models.subscription import Subscription
+from sdc_engine_helpers.date_utils import DateUtils
+from sdc_engine_helpers import decorators as engine_decorators
 from sdc_engine_helpers.sagemaker.sagemaker_query_helpers import \
     SagemakerEngineHelpers
 from sdc_engine_helpers.sagemaker.maintenance.campaign_state_manager import \
@@ -21,8 +25,6 @@ from sdc_engine_helpers.sagemaker.maintenance.solution_state_manager import \
     SolutionStateManager
 from sdc_engine_helpers.sagemaker.maintenance.dataset_state_manager import \
     DatasetStateManager
-from sdc_engine_helpers.models.subscription import Subscription
-from sdc_engine_helpers.date_utils import DateUtils
 
 class MaintenanceManager:
     """
@@ -30,7 +32,7 @@ class MaintenanceManager:
     """
     # get helpers
     date_utils = DateUtils()
-
+    slack_helper = SlackHelper()
     solution_base_name = None
 
     # set up states
@@ -111,8 +113,8 @@ class MaintenanceManager:
 
         return database_session_updated
 
-    @decorators.retry_handler(
-        exceptions=(Exception, InternalError,),
+    @engine_decorators.retry_handler(
+        exceptions=(Exception, InternalError),
         total_tries=4,
         initial_wait=0.5,
         backoff_factor=2
@@ -134,6 +136,12 @@ class MaintenanceManager:
 
         return True
 
+
+    @global_decorators.general_exception_handler(
+        logger=slack_helper,
+        exceptions=(botocore_clienterror, Exception),
+        should_raise=False
+    )
     def perform_dataset_maintenance(
             self, *,
             client: dict,
@@ -145,6 +153,7 @@ class MaintenanceManager:
             - for each dataset, check state for vendor
         """
         datasets_updated = False
+        curr_state = {}
         # get datasets
         datasets, _ = self.query_helper.get_engine_entity(
             client_id=client.id,
@@ -176,7 +185,6 @@ class MaintenanceManager:
                 for_update=True,
                 recipe=dataset['recipe']
             )
-
             # get dataset
             curr_state = self.dataset_state_manager.get_state(
                 dataset=dataset,
@@ -189,37 +197,37 @@ class MaintenanceManager:
                     'Updating dataset version for client: {client_name}'
                     .format(client_name=client.name)
                 )
-                try:
-                    new_dataset = self.dataset_state_manager.update_state(
-                        dataset=dataset,
-                        campaign=campaign
-                    )
 
-                    # update dataset
-                    self.query_helper.update_dataset(
-                        client_id=client.id,
-                        service_id=self.service.id,
-                        engine_slug=engine_slug,
-                        label=dataset['label'],
-                        config=new_dataset
-                    )
+                new_dataset = self.dataset_state_manager.update_state(
+                    dataset=dataset,
+                    campaign=campaign
+                )
 
-                    dataset_label = dataset['label']
-                    print(
-                        'Successfully updated dataset version: {dataset_label}'
-                        .format(dataset_label=dataset_label)
-                    )
+                # update dataset
+                self.query_helper.update_dataset(
+                    client_id=client.id,
+                    service_id=self.service.id,
+                    engine_slug=engine_slug,
+                    label=dataset['label'],
+                    config=new_dataset
+                )
 
-                    datasets_updated = True
-                except botocore_clienterror as exception:
-                    print((
-                        "Failed with exception = {}"
-                        "\nUnable to update database dataset "
-                        "due to vendor api failure.".format(exception)
-                    ))
+                dataset_label = dataset['label']
+                print(
+                    'Successfully updated dataset version: {dataset_label}'
+                    .format(dataset_label=dataset_label)
+                )
+
+                datasets_updated = True
 
         return datasets_updated
 
+
+    @global_decorators.general_exception_handler(
+        logger=slack_helper,
+        exceptions=(botocore_clienterror, Exception),
+        should_raise=False
+    )
     def perform_solution_maintenance(self, *, client: dict, engine_slug: str) -> dict:
         """
             Perform the required maintenance on a list of solutions
@@ -233,7 +241,7 @@ class MaintenanceManager:
 
         """
         solutions_updated = False
-
+        curr_state = {}
         # get solutions
         solutions, _ = self.query_helper.get_engine_entity(
             client_id=client.id,
@@ -272,35 +280,29 @@ class MaintenanceManager:
                     prefix=[client.name, engine_slug]
                 )
 
-                try:
-                    # train a new model if should_create_version True
-                    new_solution = self.solution_state_manager.update_state(
-                        solution=solution,
-                        solution_base_name=new_solution_base_name
-                    )
+                # train a new model if should_create_version True
+                new_solution = self.solution_state_manager.update_state(
+                    solution=solution,
+                    solution_base_name=new_solution_base_name
+                )
 
-                    # get solution arn
-                    solution_arn = new_solution.get('arn')
-                    print(
-                        'Successfully started creation of solution version: {solution_version_arn}'
-                        .format(solution_version_arn=solution_arn)
-                    )
+                # get solution arn
+                solution_arn = new_solution.get('arn')
+                print(
+                    'Successfully started creation of solution version: {solution_version_arn}'
+                    .format(solution_version_arn=solution_arn)
+                )
 
-                    # update immedidately and change status
-                    self.query_helper.update_solution(
-                        client_id=client.id,
-                        service_id=self.service.id,
-                        engine_slug=engine_slug,
-                        recipe=solution['recipe'],
-                        config=new_solution
-                    )
+                # update immedidately and change status
+                self.query_helper.update_solution(
+                    client_id=client.id,
+                    service_id=self.service.id,
+                    engine_slug=engine_slug,
+                    recipe=solution['recipe'],
+                    config=new_solution
+                )
 
-                    solutions_updated = True
-                except botocore_clienterror as exception:
-                    print((
-                        "Failed with exception = {}"
-                        "\nUnable to update vendor solution.".format(exception)
-                    ))
+                solutions_updated = True
 
             elif curr_state.get('should_refresh_version_status') is True:
                 # update state step >> solution manager
@@ -316,8 +318,7 @@ class MaintenanceManager:
                         status=new_version_status
                     )
                 )
-
-                # update state here
+                # update immedidately and change status
                 self.query_helper.update_solution(
                     client_id=client.id,
                     service_id=self.service.id,
@@ -332,7 +333,12 @@ class MaintenanceManager:
 
         return solutions_updated
 
-    # pylint: disable=too-many-locals
+
+    @global_decorators.general_exception_handler(
+        logger=slack_helper,
+        exceptions=(botocore_clienterror, Exception),
+        should_raise=False
+    )
     def perform_campaign_maintenance(
             self, *,
             client: dict,
@@ -351,6 +357,7 @@ class MaintenanceManager:
 
         """
         campaigns_updated = False
+        curr_state = {}
         # get campaigns for this engine
         campaigns, _ = self.query_helper.get_engine_entity(
             client_id=client.id,
@@ -401,29 +408,23 @@ class MaintenanceManager:
                     )
                 )
 
-                try:
-                    new_campaign = self.campaign_state_manager.update_state(
-                        campaign=campaign,
-                        solution=solution
-                    )
+                new_campaign = self.campaign_state_manager.update_state(
+                    campaign=campaign,
+                    solution=solution
+                )
 
-                    print('Successfully started updating campaign: {arn}'.format(arn=arn))
+                print('Successfully started updating campaign: {arn}'.format(arn=arn))
 
-                    # try and update else fail
-                    self.query_helper.update_campaign(
-                        client_id=client.id,
-                        service_id=self.service.id,
-                        engine_slug=engine_slug,
-                        recipe=campaign['recipe'],
-                        config=new_campaign
-                    )
+                # try and update else fail
+                self.query_helper.update_campaign(
+                    client_id=client.id,
+                    service_id=self.service.id,
+                    engine_slug=engine_slug,
+                    recipe=campaign['recipe'],
+                    config=new_campaign
+                )
 
-                    campaigns_updated = True
-                except botocore_clienterror as exception:
-                    print((
-                        "Failed with exception = {}"
-                        "\nUnable to update vendor campaign.".format(exception)
-                    ))
+                campaigns_updated = True
 
             elif curr_state.get('should_refresh_status') is True:
                 # get current version arn
@@ -441,6 +442,8 @@ class MaintenanceManager:
                         status=new_status
                     )
                 )
+                new_status = curr_state.get('new_status')
+                last_refreshed_at = curr_state.get('last_refreshed_at')
 
                 # solution refresh to account for endpoint roll-back
                 # when sagemaker endpoint update fails, it rolls back
