@@ -2,10 +2,10 @@
     Sagemaker recommendations manager module
 """
 import json
-import sdc_helpers.utils as utils
 from sdc_helpers.models.client import Client
 from sdc_helpers.redis_helper import RedisHelper
 import boto3
+from sdc_engine_helpers import queue_utils
 from sdc_engine_helpers.sagemaker.sagemaker_query_helpers import \
     SagemakerEngineHelpers
 
@@ -99,32 +99,13 @@ class RecommendationsManager:
             num_results=10
         )
 
-        domain = utils.dict_query(
-            dictionary=context,
-            path='domain'
+        queue_utils.queue_s3_recommendations_store(
+            client=self.client,
+            results=results,
+            item_id=int(item_id),
+            context=context if context is not None else {}
         )
 
-        session_hash = utils.dict_query(
-            dictionary=context,
-            path='sessionHash'
-        )
-
-        # trigger will be written with alphabetical order in mind
-        trigger_log = {
-            'trigger': {
-                'client': self.client.name,
-                'type': 'recommendations',
-                'data': {
-                    'domain': domain,
-                    'item_id': item_id,
-                    'session_hash': session_hash,
-                    'results': results
-                }
-            }
-        }
-
-        # write to lambda output logs
-        print(json.dumps(trigger_log))
         print(results)
         return results
 
